@@ -3,7 +3,7 @@ Core implementation of :mod:`facet.inspection`
 """
 
 import logging
-from typing import Generic, List, Optional, TypeVar, Union, cast
+from typing import Any, Generic, Iterable, List, Optional, Tuple, TypeVar, Union, cast
 
 import numpy as np
 import pandas as pd
@@ -17,6 +17,8 @@ from pytools.viz.dendrogram import LinkageTree
 from sklearndf import ClassifierDF, LearnerDF, RegressorDF
 from sklearndf.pipeline import LearnerPipelineDF
 
+from ..crossfit import LearnerCrossfit
+from ..data import Sample
 from ._explainer import ExplainerFactory, TreeExplainerFactory
 from ._shap import (
     ClassifierShapInteractionValuesCalculator,
@@ -26,16 +28,17 @@ from ._shap import (
     ShapCalculator,
     ShapInteractionValuesCalculator,
 )
-from facet.crossfit import LearnerCrossfit
-from facet.data import Sample
-from facet.inspection._shap_decomposition import (
-    ShapInteractionValueDecomposer,
-    ShapValueDecomposer,
+from ._shap_global_explanation import (
+    ShapGlobalExplainer,
+    ShapInteractionGlobalExplainer,
 )
+from ._shap_projection import ShapInteractionVectorProjector, ShapVectorProjector
 
 log = logging.getLogger(__name__)
 
 __all__ = ["ShapPlotData", "LearnerInspector"]
+
+
 #
 # Type variables
 #
@@ -65,7 +68,7 @@ class ShapPlotData:
 
     def __init__(
         self, shap_values: Union[np.ndarray, List[np.ndarray]], sample: Sample
-    ):
+    ) -> None:
         """
         :param shap_values: the shap values for all observations and outputs
         :param sample: (sub)sample of all observations for which SHAP values are
@@ -101,7 +104,7 @@ class ShapPlotData:
 
 @inheritdoc(match="[see superclass]")
 class LearnerInspector(
-    FittableMixin[Sample], ParallelizableMixin, Generic[T_LearnerPipelineDF]
+    FittableMixin[LearnerCrossfit], ParallelizableMixin, Generic[T_LearnerPipelineDF]
 ):
     """
     Explain regressors and classifiers based on SHAP values.
@@ -122,8 +125,8 @@ class LearnerInspector(
       values)
     - pairwise feature synergy matrix (requires availability of SHAP interaction
       values)
-    - pairwise feature association matrix (combined effect of redundancy and
-      synergy, if no SHAP interaction values are available)
+    - pairwise feature association matrix (upper bound for redundancy but can be
+      inflated by synergy; available if SHAP interaction values are unknown)
     - feature redundancy linkage (to visualize clusters of redundant features in a
       dendrogram)
     - feature synergy linkage (to visualize clusters of synergistic features in a
@@ -135,6 +138,14 @@ class LearnerInspector(
     specified in the underlying training sample.
     """
 
+    #: constant for "mean" aggregation method, to be passed as arg ``aggregation``
+    #: to :class:`.LearnerInspector` methods that implement it
+    AGG_MEAN = "mean"
+
+    #: constant for "std" aggregation method, to be passed as arg ``aggregation``
+    #: to :class:`.LearnerInspector` methods that implement it
+    AGG_STD = "std"
+
     #: Name for feature importance series or column.
     COL_IMPORTANCE = "importance"
 
@@ -150,7 +161,6 @@ class LearnerInspector(
         *,
         explainer_factory: Optional[ExplainerFactory] = None,
         shap_interaction: bool = True,
-        min_direct_synergy: Optional[float] = None,
         n_jobs: Optional[int] = None,
         shared_memory: Optional[bool] = None,
         pre_dispatch: Optional[Union[str, int]] = None,
@@ -164,14 +174,6 @@ class LearnerInspector(
             SHAP interaction values are needed to determine feature synergy and
             redundancy.
             (default: ``True``)
-        :param min_direct_synergy: minimum direct synergy to consider a feature pair
-            for calculation of indirect synergy,
-            only relevant if parameter ``shap_interaction`` is ``True``.
-            This effectively acts as a noise threshold given the approximate nature of
-            SHAP calculations.
-            Consider increasing this value if you see warnings about contravariant
-            indirect synergy when fitting this inspector.
-            (default: <DEFAULT_MIN_DIRECT_SYNERGY>)
         """
         super().__init__(
             n_jobs=n_jobs,
@@ -189,33 +191,26 @@ class LearnerInspector(
             explainer_factory = self.DEFAULT_EXPLAINER_FACTORY
             assert explainer_factory.explains_raw_output
 
-        if shap_interaction and not explainer_factory.supports_shap_interaction_values:
-            log.warning(
-                "ignoring arg shap_interaction=True: "
-                "explainers made by arg explainer_factory do not support "
-                "SHAP interaction values"
-            )
-            shap_interaction = False
+        if shap_interaction:
+            if not explainer_factory.supports_shap_interaction_values:
+                log.warning(
+                    "ignoring arg shap_interaction=True: "
+                    "explainers made by arg explainer_factory do not support "
+                    "SHAP interaction values"
+                )
+                shap_interaction = False
 
         self._explainer_factory = explainer_factory
         self._shap_interaction = shap_interaction
-        self._min_direct_synergy = min_direct_synergy
 
         self._crossfit: Optional[LearnerCrossfit[T_LearnerPipelineDF]] = None
         self._shap_calculator: Optional[ShapCalculator] = None
-        self._shap_decomposer: Optional[ShapValueDecomposer] = None
+        self._shap_global_decomposer: Optional[ShapGlobalExplainer] = None
+        self._shap_global_projector: Optional[ShapGlobalExplainer] = None
 
-    # dynamically complete the __init__ docstring
-    # noinspection PyTypeChecker
-    __init__.__doc__ = (
-        __init__.__doc__.replace(
-            "<DEFAULT_MIN_DIRECT_SYNERGY>",
-            str(ShapInteractionValueDecomposer.DEFAULT_MIN_DIRECT_SYNERGY),
-        )
-        + ParallelizableMixin.__init__.__doc__
-    )
+    __init__.__doc__ += ParallelizableMixin.__init__.__doc__
 
-    def fit(self: T_Self, crossfit: LearnerCrossfit, **fit_params) -> T_Self:
+    def fit(self: T_Self, crossfit: LearnerCrossfit, **fit_params: Any) -> T_Self:
         """
         Fit the inspector with the given crossfit.
 
@@ -227,6 +222,10 @@ class LearnerInspector(
             compatibility with :class:`.FittableMixin`)
         :return: ``self``
         """
+        # :param full_sample: if ``True``, explain only a single model fitted on the
+        # full sample; otherwise, explain all models in the crossfit and aggregate
+        # results
+        full_sample = bool(fit_params.get("full_sample", False))
 
         self: LearnerInspector  # support type hinting in PyCharm
 
@@ -255,6 +254,10 @@ class LearnerInspector(
                 f"but is a {type(learner).__name__}"
             )
 
+        shap_global_projector: Union[
+            ShapVectorProjector, ShapInteractionVectorProjector, None
+        ]
+
         if self._shap_interaction:
             shap_calculator_type = (
                 ClassifierShapInteractionValuesCalculator
@@ -262,16 +265,15 @@ class LearnerInspector(
                 else RegressorShapInteractionValuesCalculator
             )
             shap_calculator = shap_calculator_type(
-                explain_full_sample=False,
+                explain_full_sample=full_sample,
                 explainer_factory=self._explainer_factory,
                 n_jobs=self.n_jobs,
                 shared_memory=self.shared_memory,
                 pre_dispatch=self.pre_dispatch,
                 verbose=self.verbose,
             )
-            shap_decomposer = ShapInteractionValueDecomposer(
-                min_direct_synergy=self._min_direct_synergy
-            )
+
+            shap_global_projector = ShapInteractionVectorProjector()
 
         else:
             shap_calculator_type = (
@@ -280,24 +282,30 @@ class LearnerInspector(
                 else RegressorShapValuesCalculator
             )
             shap_calculator = shap_calculator_type(
-                explain_full_sample=False,
+                explain_full_sample=full_sample,
                 explainer_factory=self._explainer_factory,
                 n_jobs=self.n_jobs,
                 shared_memory=self.shared_memory,
                 pre_dispatch=self.pre_dispatch,
                 verbose=self.verbose,
             )
-            shap_decomposer = ShapValueDecomposer()
+
+            shap_global_projector = ShapVectorProjector()
 
         shap_calculator.fit(crossfit=crossfit)
-        shap_decomposer.fit(shap_calculator=shap_calculator)
+        shap_global_projector.fit(shap_calculator=shap_calculator)
 
         self._shap_calculator = shap_calculator
-        self._shap_decomposer = shap_decomposer
+        self._shap_global_projector = shap_global_projector
+
         self._crossfit = crossfit
 
         return self
 
+    @property
+    def _shap_global_explainer(self) -> ShapGlobalExplainer:
+        return self._shap_global_projector
+
     @property
     def is_fitted(self) -> bool:
         """[see superclass]"""
@@ -337,7 +345,7 @@ class LearnerInspector(
         return self._shap_calculator.output_names_
 
     @property
-    def features(self) -> List[str]:
+    def features_(self) -> List[str]:
         """
         The names of the features used to fit the learner pipeline explained by this
         inspector.
@@ -345,7 +353,7 @@ class LearnerInspector(
         return self.crossfit_.pipeline.feature_names_out_.to_list()
 
     def shap_values(
-        self, consolidate: Optional[str] = "mean"
+        self, aggregation: Optional[str] = AGG_MEAN
     ) -> Union[pd.DataFrame, List[pd.DataFrame]]:
         """
         Calculate the SHAP values for all observations and features.
@@ -356,31 +364,31 @@ class LearnerInspector(
         By default, one SHAP value is returned for each observation and feature; this
         value is calculated as the mean SHAP value across all crossfits.
 
-        The ``consolidate`` argument can be used to disable or change the consolidation
+        The ``aggregation`` argument can be used to disable or change the aggregation
         of SHAP values:
 
-        - passing ``consolidate=None`` will disable SHAP value consolidation,
+        - passing ``aggregation=None`` will disable SHAP value aggregation,
           generating one row for every crossfit and observation (identified by
           a hierarchical index with two levels)
-        - passing ``consolidate="mean"`` (the default) will calculate the mean SHAP
+        - passing ``aggregation="mean"`` (the default) will calculate the mean SHAP
           values across all crossfits
-        - passing ``consolidate="std"`` will calculate the standard deviation of SHAP
+        - passing ``aggregation="std"`` will calculate the standard deviation of SHAP
           values across all crossfits, as the basis for determining the uncertainty
           of SHAP calculations
 
-        :param consolidate: consolidate SHAP values across splits;
+        :param aggregation: aggregation SHAP values across splits;
             permissible values are ``"mean"`` (calculate the mean), ``"std"``
-            (calculate the standard deviation), or ``None`` to prevent consolidation
+            (calculate the standard deviation), or ``None`` to prevent aggregation
             (default: ``"mean"``)
         :return: a data frame with SHAP values
         """
         self._ensure_fitted()
         return self.__split_multi_output_df(
-            self._shap_calculator.get_shap_values(consolidate=consolidate)
+            self._shap_calculator.get_shap_values(aggregation=aggregation)
         )
 
     def shap_interaction_values(
-        self, consolidate: Optional[str] = "mean"
+        self, aggregation: Optional[str] = AGG_MEAN
     ) -> Union[pd.DataFrame, List[pd.DataFrame]]:
         """
         Calculate the SHAP interaction values for all observations and pairs of
@@ -394,28 +402,28 @@ class LearnerInspector(
         feature pairing; this value is calculated as the mean SHAP interaction value
         across all crossfits.
 
-        The ``consolidate`` argument can be used to disable or change the consolidation
+        The ``aggregation`` argument can be used to disable or change the aggregation
         of SHAP interaction values:
 
-        - passing ``consolidate=None`` will disable SHAP interaction value
-          consolidation, generating one row for every crossfit, observation and
+        - passing ``aggregation=None`` will disable SHAP interaction value
+          aggregation, generating one row for every crossfit, observation and
           feature (identified by a hierarchical index with three levels)
-        - passing ``consolidate="mean"`` (the default) will calculate the mean SHAP
+        - passing ``aggregation="mean"`` (the default) will calculate the mean SHAP
           interaction values across all crossfits
-        - passing ``consolidate="std"`` will calculate the standard deviation of SHAP
+        - passing ``aggregation="std"`` will calculate the standard deviation of SHAP
           interaction values across all crossfits, as the basis for determining the
           uncertainty of SHAP calculations
 
-        :param consolidate: consolidate SHAP interaction values across splits;
+        :param aggregation: aggregate SHAP interaction values across splits;
             permissible values are ``"mean"`` (calculate the mean), ``"std"``
-            (calculate the standard deviation), or ``None`` to prevent consolidation
+            (calculate the standard deviation), or ``None`` to prevent aggregation
             (default: ``"mean"``)
         :return: a data frame with SHAP interaction values
         """
         self._ensure_fitted()
         return self.__split_multi_output_df(
             self.__shap_interaction_values_calculator.get_shap_interaction_values(
-                consolidate=consolidate
+                aggregation=aggregation
             )
         )
 
@@ -444,7 +452,7 @@ class LearnerInspector(
             raise ValueError(f'arg method="{method}" must be one of {methods}')
 
         shap_matrix: pd.DataFrame = self._shap_calculator.get_shap_values(
-            consolidate="mean"
+            aggregation="mean"
         )
         weight: Optional[pd.Series] = self.sample_.weight
 
@@ -477,7 +485,12 @@ class LearnerInspector(
             return _normalize_importance(abs_importance.unstack(level=0))
 
     def feature_synergy_matrix(
-        self, symmetrical: bool = False, clustered: bool = True
+        self,
+        *,
+        absolute: bool = False,
+        symmetrical: bool = False,
+        aggregation: Optional[str] = AGG_MEAN,
+        clustered: bool = True,
     ) -> Union[pd.DataFrame, List[pd.DataFrame]]:
         """
         Calculate the feature synergy matrix.
@@ -497,10 +510,15 @@ class LearnerInspector(
         In the case of multi-target regression and non-binary classification, returns
         a list of data frames with one matrix per output.
 
+        :param absolute: if ``False``, return relative synergy as a percentage of
+            total feature importance;
+            if ``True``, return absolute synergy as a portion of feature importance
         :param symmetrical: if ``True``, return a symmetrical matrix quantifying
             mutual synergy; if ``False``, return an asymmetrical matrix quantifying
-            unilateral redundancy of the features represented by rows with the
+            unilateral synergy of the features represented by rows with the
             features represented by columns (default: ``False``)
+        :param aggregation: if ``mean``, return mean values across all models in the
+            crossfit; additional aggregation methods will be added in future releases
         :param clustered: if ``True``, reorder the rows and columns of the matrix
             such that synergy between adjacent rows and columns is maximised; if
             ``False``, keep rows and columns in the original features order
@@ -510,16 +528,28 @@ class LearnerInspector(
         """
         self._ensure_fitted()
 
+        self.__validate_aggregation_method(aggregation)
+
+        explainer = self.__interaction_explainer
         return self.__feature_affinity_matrix(
             affinity_matrices=(
-                self.__shap_interaction_decomposer.synergy(symmetrical=symmetrical)
+                explainer.to_frames(
+                    explainer.synergy(symmetrical=symmetrical, absolute=absolute)
+                )
+            ),
+            affinity_symmetrical=explainer.synergy(
+                symmetrical=True, absolute=False, std=False
             ),
-            affinity_symmetrical=self.__shap_interaction_decomposer.synergy_rel_,
             clustered=clustered,
         )
 
     def feature_redundancy_matrix(
-        self, symmetrical: bool = False, clustered: bool = True
+        self,
+        *,
+        absolute: bool = False,
+        symmetrical: bool = False,
+        aggregation: Optional[str] = AGG_MEAN,
+        clustered: bool = True,
     ) -> Union[pd.DataFrame, List[pd.DataFrame]]:
         """
         Calculate the feature redundancy matrix.
@@ -539,10 +569,15 @@ class LearnerInspector(
         In the case of multi-target regression and non-binary classification, returns
         a list of data frames with one matrix per output.
 
+        :param absolute: if ``False``, return relative redundancy as a percentage of
+            total feature importance;
+            if ``True``, return absolute redundancy as a portion of feature importance
         :param symmetrical: if ``True``, return a symmetrical matrix quantifying
             mutual redundancy; if ``False``, return an asymmetrical matrix quantifying
             unilateral redundancy of the features represented by rows with the
             features represented by columns (default: ``False``)
+        :param aggregation: if ``mean``, return mean values across all models in the
+            crossfit; additional aggregation methods will be added in future releases
         :param clustered: if ``True``, reorder the rows and columns of the matrix
             such that redundancy between adjacent rows and columns is maximised; if
             ``False``, keep rows and columns in the original features order
@@ -552,16 +587,28 @@ class LearnerInspector(
         """
         self._ensure_fitted()
 
+        self.__validate_aggregation_method(aggregation)
+
+        explainer = self.__interaction_explainer
         return self.__feature_affinity_matrix(
             affinity_matrices=(
-                self.__shap_interaction_decomposer.redundancy(symmetrical=symmetrical)
+                explainer.to_frames(
+                    explainer.redundancy(symmetrical=symmetrical, absolute=absolute)
+                )
+            ),
+            affinity_symmetrical=explainer.redundancy(
+                symmetrical=True, absolute=False, std=False
             ),
-            affinity_symmetrical=self.__shap_interaction_decomposer.redundancy_rel_,
             clustered=clustered,
         )
 
     def feature_association_matrix(
-        self, symmetrical: bool = False, clustered: bool = True
+        self,
+        *,
+        absolute: bool = False,
+        symmetrical: bool = False,
+        aggregation: Optional[str] = AGG_MEAN,
+        clustered: bool = True,
     ) -> Union[pd.DataFrame, List[pd.DataFrame]]:
         """
         Calculate the feature association matrix.
@@ -572,9 +619,8 @@ class LearnerInspector(
 
         The association of a feature with itself is defined as `1.0`.
 
-        Feature association is the combined effect of feature redundancy and feature
-        synergy (see methods :meth:`.feature_redundancy_matrix` and
-        :meth:`feature_synergy_matrix`).
+        Feature association provides an upper bound for feature redundancy but might be
+        inflated by feature synergy.
 
         While it is preferable to assess redundancy and synergy separately, association
         can be calculated using only SHAP values, and thus can be used as a fallback
@@ -583,10 +629,16 @@ class LearnerInspector(
         In the case of multi-target regression and non-binary classification, returns
         a list of data frames with one matrix per output.
 
-        :param symmetrical: if ``True``, return a symmetrical matrix quantifying
-            mutual association; if ``False``, return an asymmetrical matrix
+        :param absolute: if ``False``, return relative association as a percentage of
+            total feature importance;
+            if ``True``, return absolute association as a portion of feature importance
+        :param symmetrical: if ``False``, return an asymmetrical matrix
             quantifying unilateral association of the features represented by rows
-            with the features represented by columns (default: ``False``)
+            with the features represented by columns;
+            if ``True``, return a symmetrical matrix quantifying mutual association
+            (default: ``False``)
+        :param aggregation: if ``mean``, return mean values across all models in the
+            crossfit; additional aggregation methods will be added in future releases
         :param clustered: if ``True``, reorder the rows and columns of the matrix
             such that association between adjacent rows and columns is maximised; if
             ``False``, keep rows and columns in the original features order
@@ -596,11 +648,20 @@ class LearnerInspector(
         """
         self._ensure_fitted()
 
+        self.__validate_aggregation_method(aggregation)
+
+        global_explainer = self._shap_global_explainer
         return self.__feature_affinity_matrix(
             affinity_matrices=(
-                self._shap_decomposer.association(symmetrical=symmetrical)
+                global_explainer.to_frames(
+                    global_explainer.association(
+                        absolute=absolute, symmetrical=symmetrical
+                    )
+                )
+            ),
+            affinity_symmetrical=global_explainer.association(
+                symmetrical=True, absolute=False, std=False
             ),
-            affinity_symmetrical=self._shap_decomposer.association_rel_,
             clustered=clustered,
         )
 
@@ -619,7 +680,9 @@ class LearnerInspector(
         """
         self._ensure_fitted()
         return self.__linkages_from_affinity_matrices(
-            feature_affinity_matrix=self.__shap_interaction_decomposer.synergy_rel_
+            feature_affinity_matrix=self.__interaction_explainer.synergy(
+                symmetrical=True, absolute=False, std=False
+            )
         )
 
     def feature_redundancy_linkage(self) -> Union[LinkageTree, List[LinkageTree]]:
@@ -637,7 +700,9 @@ class LearnerInspector(
         """
         self._ensure_fitted()
         return self.__linkages_from_affinity_matrices(
-            feature_affinity_matrix=self.__shap_interaction_decomposer.redundancy_rel_
+            feature_affinity_matrix=self.__interaction_explainer.redundancy(
+                symmetrical=True, absolute=False, std=False
+            )
         )
 
     def feature_association_linkage(self) -> Union[LinkageTree, List[LinkageTree]]:
@@ -655,7 +720,9 @@ class LearnerInspector(
         """
         self._ensure_fitted()
         return self.__linkages_from_affinity_matrices(
-            feature_affinity_matrix=self._shap_decomposer.association_rel_
+            feature_affinity_matrix=self._shap_global_explainer.association(
+                absolute=False, symmetrical=True, std=False
+            )
         )
 
     def feature_interaction_matrix(self) -> Union[pd.DataFrame, List[pd.DataFrame]]:
@@ -705,22 +772,20 @@ class LearnerInspector(
         with :math:`\\sum_{a=1}^n \\sum_{b=a}^n I_{ab} = 1`
 
         In the case of multi-target regression and non-binary classification, returns
-        a data frame with one matrix per output, stacked horizontally, and with a
-        hierarchical column index (target/class name on level 1, and feature name on
-        level 2).
+        a list with one matrix per output.
 
         :return: relative shap interaction values as a data frame of shape
-            (n_features, n_outputs * n_features)
+            `(n_features, n_features)`; or a list of such data frames
         """
 
-        n_features = len(self.features)
+        n_features = len(self.features_)
         n_outputs = len(self.output_names_)
 
         # get a feature interaction array with shape
         # (n_observations, n_outputs, n_features, n_features)
         # where the innermost feature x feature arrays are symmetrical
         im_matrix_per_observation_and_output = (
-            self.shap_interaction_values(consolidate=None)
+            self.shap_interaction_values(aggregation=None)
             .values.reshape((-1, n_features, n_outputs, n_features))
             .swapaxes(1, 2)
         )
@@ -794,7 +859,7 @@ class LearnerInspector(
         """
 
         shap_values: Union[pd.DataFrame, List[pd.DataFrame]] = self.shap_values(
-            consolidate="mean"
+            aggregation="mean"
         )
 
         output_names: List[str] = self.output_names_
@@ -917,6 +982,11 @@ class LearnerInspector(
             )
 
         else:
+            # noinspection PyCompatibility
+            feature_importance_iter: (
+                Iterable[Tuple[Any, pd.Series]]
+            ) = feature_importance.iteritems()
+
             return [
                 self.__linkage_tree_from_affinity_matrix_for_output(
                     feature_affinity_for_output, feature_importance_for_output
@@ -924,7 +994,7 @@ class LearnerInspector(
                 for feature_affinity_for_output, (
                     _,
                     feature_importance_for_output,
-                ) in zip(feature_affinity_matrix, feature_importance.iteritems())
+                ) in zip(feature_affinity_matrix, feature_importance_iter)
             ]
 
     @staticmethod
@@ -989,15 +1059,20 @@ class LearnerInspector(
         else:
             return frames
 
+    @staticmethod
+    def __validate_aggregation_method(aggregation: str) -> None:
+        if aggregation != LearnerInspector.AGG_MEAN:
+            raise ValueError(f"unknown aggregation method: aggregation={aggregation}")
+
     @property
     def __shap_interaction_values_calculator(self) -> ShapInteractionValuesCalculator:
         self._ensure_shap_interaction()
         return cast(ShapInteractionValuesCalculator, self._shap_calculator)
 
     @property
-    def __shap_interaction_decomposer(self) -> ShapInteractionValueDecomposer:
+    def __interaction_explainer(self) -> ShapInteractionGlobalExplainer:
         self._ensure_shap_interaction()
-        return cast(ShapInteractionValueDecomposer, self._shap_decomposer)
+        return cast(ShapInteractionGlobalExplainer, self._shap_global_explainer)
 
 
 __tracker.validate()
