@@ -8,9 +8,11 @@ from typing import (
     Any,
     Callable,
     Generic,
+    Iterator,
     List,
     Optional,
     Sequence,
+    Tuple,
     Type,
     TypeVar,
     Union,
@@ -18,6 +20,7 @@ from typing import (
 
 import numpy as np
 import pandas as pd
+from scipy import stats
 
 from pytools.api import AllTracker, inheritdoc
 from pytools.parallelization import Job, JobRunner, ParallelizableMixin
@@ -43,6 +46,7 @@ __all__ = [
     "UnivariateUpliftSimulator",
 ]
 
+
 #
 # Type variables
 #
@@ -50,6 +54,7 @@ __all__ = [
 T_LearnerPipelineDF = TypeVar("T_LearnerPipelineDF", bound=LearnerPipelineDF)
 T_Partition = TypeVar("T_Partition")
 
+
 #
 # Ensure all symbols introduced below are included in __all__
 #
@@ -127,9 +132,12 @@ class UnivariateSimulationResult(Generic[T_Partition]):
         """
         super().__init__()
 
-        assert (
-            outputs.index.name == BaseUnivariateSimulator.IDX_SPLIT
-        ), f"row index of arg outputs is named {BaseUnivariateSimulator.IDX_SPLIT}"
+        assert outputs.index.name in [
+            BaseUnivariateSimulator.IDX_SPLIT,
+            # for the experimental _full sample_ feature, we also accept "metric" as
+            # the name of the row index
+            "metric",
+        ], f"row index of arg outputs is named {BaseUnivariateSimulator.IDX_SPLIT}"
         assert outputs.columns.name == BaseUnivariateSimulator.IDX_PARTITION, (
             "column index of arg outputs is named "
             f"{BaseUnivariateSimulator.IDX_PARTITION}"
@@ -154,7 +162,14 @@ class UnivariateSimulationResult(Generic[T_Partition]):
         :return: a series of medians, indexed by the central values of the partitions
             for which the simulation was run
         """
-        return self.outputs.median().rename(UnivariateSimulationResult.COL_MEDIAN)
+        if self._full_sample:
+            # experimental feature: we only simulated using one model fit on the full
+            # sample; return the mean outputs for each partition without aggregating
+            # further
+            values = self.outputs.loc["mean"]
+        else:
+            values = self.outputs.median()
+        return values.rename(UnivariateSimulationResult.COL_MEDIAN)
 
     def outputs_lower_bound(self) -> pd.Series:
         """
@@ -164,9 +179,19 @@ class UnivariateSimulationResult(Generic[T_Partition]):
         :return: a series of medians, indexed by the central values of the partitions
             for which the simulation was run
         """
-        return self.outputs.quantile(q=(1.0 - self.confidence_level) / 2.0).rename(
-            UnivariateSimulationResult.COL_LOWER_BOUND
-        )
+        if self._full_sample:
+            # experimental feature: we only simulated using one model fit on the full
+            # sample; return the mean outputs for each partition without aggregating
+            # further, and determine the lower confidence bound based on the standard
+            # error of the mean and the desired confidence level
+            values = (
+                self.outputs.loc["mean"]
+                + stats.norm.ppf((1.0 - self.confidence_level) / 2.0)
+                * self.outputs.loc["sem"]
+            )
+        else:
+            values = self.outputs.quantile(q=(1.0 - self.confidence_level) / 2.0)
+        return values.rename(UnivariateSimulationResult.COL_LOWER_BOUND)
 
     def outputs_upper_bound(self) -> pd.Series:
         """
@@ -176,9 +201,25 @@ class UnivariateSimulationResult(Generic[T_Partition]):
         :return: a series of medians, indexed by the central values of the partitions
             for which the simulation was run
         """
-        return self.outputs.quantile(
-            q=1.0 - (1.0 - self.confidence_level) / 2.0
-        ).rename(UnivariateSimulationResult.COL_UPPER_BOUND)
+        if self._full_sample:
+            # experimental feature: we only simulated using one model fit on the full
+            # sample; return the mean outputs for each partition without aggregating
+            # further, and determine the upper confidence bound based on the standard
+            # error of the mean and the desired confidence level
+            values = (
+                self.outputs.loc["mean"]
+                - stats.norm.ppf((1.0 - self.confidence_level) / 2.0)
+                * self.outputs.loc["sem"]
+            )
+        else:
+            values = self.outputs.quantile(q=1.0 - (1.0 - self.confidence_level) / 2.0)
+        return values.rename(UnivariateSimulationResult.COL_UPPER_BOUND)
+
+    @property
+    def _full_sample(self) -> bool:
+        # experimental _full sample_ feature is active iff the name of the row index
+        # is "metric"
+        return self.outputs.index.name == "metric"
 
 
 class BaseUnivariateSimulator(
@@ -198,10 +239,22 @@ class BaseUnivariateSimulator(
     #: The name of a series of simulated outputs.
     COL_OUTPUT = "output"
 
+    #: The crossfit used to conduct simulations
+    crossfit: LearnerCrossfit[T_LearnerPipelineDF]
+
+    #: The sample used in baseline calculations and simulations; this is the full sample
+    #: from the :attr:`.crossfit`, or a subsample thereof
+    sample: Sample
+
+    #: The width of the confidence interval used to calculate the lower/upper bound
+    #: of the simulation
+    confidence_level: float
+
     def __init__(
         self,
         crossfit: LearnerCrossfit[T_LearnerPipelineDF],
         *,
+        subsample: Optional[pd.Index] = None,
         confidence_level: float = 0.95,
         n_jobs: Optional[int] = None,
         shared_memory: Optional[bool] = None,
@@ -211,6 +264,8 @@ class BaseUnivariateSimulator(
         """
         :param crossfit: cross-validated crossfit of a model for all observations
             in a given sample
+        :param subsample: an optional index referencing a subset of the training sample
+            to be used in baseline calculations and simulations
         :param confidence_level: the width :math:`\\alpha` of the confidence interval
             determined by bootstrapping, with :math:`0 < \\alpha < 1`;
             for reliable CI estimates the number of splits in the crossfit should be
@@ -256,14 +311,29 @@ class BaseUnivariateSimulator(
                 f"but arg crossfit.cv has only {len(crossfit)} splits"
             )
 
+        sample = crossfit.sample_
+
+        if subsample is not None:
+            unknown_observations = subsample.difference(sample.index)
+            if len(unknown_observations) > 0:
+                raise ValueError(
+                    "arg subsample includes indices not contained "
+                    f"in the simulation sample: {unknown_observations.to_list()}"
+                )
+            sample = sample.subsample(loc=subsample)
+
         self.crossfit = crossfit
+        self.sample = sample
         self.confidence_level = confidence_level
 
     # add parallelization parameters to __init__ docstring
     __init__.__doc__ += ParallelizableMixin.__init__.__doc__
 
     def simulate_feature(
-        self, feature_name: str, *, partitioner: Partitioner[T_Partition]
+        self,
+        feature_name: str,
+        *,
+        partitioner: Partitioner[T_Partition],
     ) -> UnivariateSimulationResult:
         """
         Simulate the average target uplift when fixing the value of the given feature
@@ -274,10 +344,7 @@ class BaseUnivariateSimulator(
         :return: a mapping of output names to simulation results
         """
 
-        sample = self.crossfit.sample_
-
-        if isinstance(sample.target_name, list):
-            raise NotImplementedError("multi-output simulations are not supported")
+        sample = self.sample
 
         return UnivariateSimulationResult(
             feature_name=feature_name,
@@ -313,7 +380,6 @@ class BaseUnivariateSimulator(
         :return: series mapping split IDs to deviations of simulated mean outputs
         """
 
-        sample = self.crossfit.sample_
         y_mean = self.expected_output()
 
         result: List[float] = JobRunner.from_parallelizable(self).run_jobs(
@@ -321,10 +387,7 @@ class BaseUnivariateSimulator(
                 Job.delayed(self._simulate_actuals)(
                     model, subsample.features, y_mean, self._simulate
                 )
-                for (model, (_, test_indices)) in zip(
-                    self.crossfit.models(), self.crossfit.splits()
-                )
-                for subsample in (sample.subsample(iloc=test_indices),)
+                for model, subsample in self._get_simulations()
             )
         )
 
@@ -365,11 +428,13 @@ class BaseUnivariateSimulator(
 
     @staticmethod
     @abstractmethod
-    def _simulate(model: T_LearnerPipelineDF, x: pd.DataFrame) -> float:
+    def _simulate(model: T_LearnerPipelineDF, x: pd.DataFrame) -> pd.Series:
         pass
 
     def _simulate_feature_with_values(
-        self, feature_name: str, simulation_values: Sequence[T_Partition]
+        self,
+        feature_name: str,
+        simulation_values: Sequence[T_Partition],
     ) -> pd.DataFrame:
         """
         Run a simulation on a feature.
@@ -379,55 +444,123 @@ class BaseUnivariateSimulator(
 
         :param feature_name: name of the feature to use in the simulation
         :param simulation_values: values to use in the simulation
-        :return: data frame with three columns: ``crossfit_id``, ``parameter_value`` and
-          ``simulation_result``.
+        :return: data frame with splits as rows and partitions as columns.
         """
 
-        sample = self.crossfit.sample_
+        if feature_name not in self.sample.features.columns:
+            raise ValueError(f"feature not in sample: {feature_name}")
 
-        if feature_name not in sample.features.columns:
-            raise ValueError(f"Feature '{feature_name}' not in sample")
-
-        simulation_results_per_split: List[np.ndarray] = JobRunner.from_parallelizable(
-            self
-        ).run_jobs(
+        # for each split, calculate the mean simulation outputs and the standard error
+        # of each mean
+        simulation_means_and_sems_per_split: List[
+            Tuple[Sequence[float], Sequence[float]]
+        ] = JobRunner.from_parallelizable(self).run_jobs(
             *(
                 Job.delayed(UnivariateUpliftSimulator._simulate_values_for_split)(
                     model=model,
-                    subsample=sample.subsample(iloc=test_indices),
+                    subsample=subsample,
                     feature_name=feature_name,
                     simulated_values=simulation_values,
                     simulate_fn=self._simulate,
                 )
-                for (model, (_, test_indices)) in zip(
-                    self.crossfit.models(), self.crossfit.splits()
-                )
+                for (model, subsample) in self._get_simulations()
             )
         )
 
+        index_name: str
+        index: Optional[List[str]]
+        simulation_results_per_split: List[List[float]]
+
+        if self._full_sample:
+            # experimental "full sample" feature: we only worked with one split
+            # (which is the full sample); for that split we preserve the means and
+            # standard errors of the means for each partition
+            assert len(simulation_means_and_sems_per_split) == 1
+            simulation_results_per_split = [
+                # convert mean and sem tuple to a list
+                list(seq_result)
+                for seq_result in simulation_means_and_sems_per_split[0]
+            ]
+            index_name = "metric"
+            index = ["mean", "sem"]
+        else:
+            # existing approach: only keep the means for each split
+            simulation_results_per_split = [
+                list(seq_mean) for seq_mean, _ in simulation_means_and_sems_per_split
+            ]
+            index_name = BaseUnivariateSimulator.IDX_SPLIT
+            index = None
+
         return pd.DataFrame(
-            simulation_results_per_split, columns=simulation_values
+            simulation_results_per_split, columns=simulation_values, index=index
         ).rename_axis(
-            index=BaseUnivariateSimulator.IDX_SPLIT,
+            index=index_name,
             columns=BaseUnivariateSimulator.IDX_PARTITION,
         )
 
+    def _get_simulations(self) -> Iterator[Tuple[T_LearnerPipelineDF, Sample]]:
+        sample = self.sample
+        # we don't need duplicate indices to calculate the intersection
+        # with the samples of the test split, so we drop them
+        sample_index = sample.index.unique()
+
+        if self._full_sample:
+            # experimental flag: if `True`, simulate on full sample using all data
+            xf_sample: Sample = self.crossfit.sample_
+            return iter(
+                (
+                    (
+                        self.crossfit.pipeline.clone().fit(
+                            X=xf_sample.features,
+                            y=xf_sample.target,
+                            sample_weight=xf_sample.weight,
+                        ),
+                        sample,
+                    ),
+                )
+            )
+
+        xf_sample_index = self.crossfit.sample_.index
+        return (
+            (model, subsample)
+            for model, subsample in zip(
+                self.crossfit.models(),
+                (
+                    (
+                        sample.subsample(
+                            loc=sample_index.intersection(xf_sample_index[test_indices])
+                        )
+                    )
+                    for _, test_indices in self.crossfit.splits()
+                ),
+            )
+            if len(subsample)
+        )
+
+    @property
+    def _full_sample(self) -> Sample:
+        # experimental flag: if `True`, simulate on full sample using all data
+        full_sample = getattr(self, "full_sample", False)
+        return full_sample
+
     @staticmethod
     def _simulate_values_for_split(
         model: LearnerDF,
         subsample: Sample,
         feature_name: str,
         simulated_values: Optional[Sequence[Any]],
-        simulate_fn: Callable[[LearnerDF, pd.DataFrame], float],
-    ) -> np.ndarray:
-        # for a list of values to be simulated, return a list of absolute target changes
+        simulate_fn: Callable[[LearnerDF, pd.DataFrame], pd.Series],
+    ) -> Tuple[Sequence[float], Sequence[float]]:
+        # for a list of values to be simulated, return a sequence of mean outputs
+        # and a sequence of standard errors of those means
 
         n_observations = len(subsample)
         features = subsample.features
         feature_dtype = features.loc[:, feature_name].dtype
 
-        return np.array(
-            [
+        outputs_mean_sem: List[Tuple[float, float]] = [
+            (outputs_sr.mean(), outputs_sr.sem())
+            for outputs_sr in (
                 simulate_fn(
                     model,
                     features.assign(
@@ -441,17 +574,19 @@ class BaseUnivariateSimulator(
                     ),
                 )
                 for value in simulated_values
-            ]
-        )
+            )
+        ]
+        outputs_mean, outputs_sem = zip(*outputs_mean_sem)
+        return outputs_mean, outputs_sem
 
     @staticmethod
     def _simulate_actuals(
         model: LearnerDF,
         x: pd.DataFrame,
         y_mean: float,
-        simulate_fn: Callable[[LearnerDF, pd.DataFrame], float],
-    ):
-        return simulate_fn(model, x) - y_mean
+        simulate_fn: Callable[[LearnerDF, pd.DataFrame], pd.Series],
+    ) -> float:
+        return simulate_fn(model, x).mean() - y_mean
 
 
 @inheritdoc(match="[see superclass]")
@@ -493,10 +628,11 @@ class UnivariateProbabilitySimulator(BaseUnivariateSimulator[ClassifierPipelineD
 
         :return: observed frequency of the positive class
         """
-        actual_outputs: pd.Series = self.crossfit.sample_.target
-        assert isinstance(actual_outputs, pd.Series), "sample has one single target"
+        actual_outputs = self.sample.target.loc[self.subsample]
 
-        return (actual_outputs == self._positive_class()).sum() / len(actual_outputs)
+        return actual_outputs.loc[actual_outputs == self._positive_class()].sum() / len(
+            actual_outputs
+        )
 
     def _positive_class(self) -> Any:
         """
@@ -518,11 +654,11 @@ class UnivariateProbabilitySimulator(BaseUnivariateSimulator[ClassifierPipelineD
         return ClassifierPipelineDF
 
     @staticmethod
-    def _simulate(model: ClassifierPipelineDF, x: pd.DataFrame) -> float:
+    def _simulate(model: ClassifierPipelineDF, x: pd.DataFrame) -> pd.Series:
         probabilities: pd.DataFrame = model.predict_proba(x)
         if probabilities.shape[1] != 2:
             raise TypeError("only binary classifiers are supported")
-        return probabilities.iloc[:, 1].mean()
+        return probabilities.iloc[:, 1]
 
 
 class _UnivariateRegressionSimulator(
@@ -534,18 +670,17 @@ class _UnivariateRegressionSimulator(
 
         :return: mean observed value of the target
         """
-        actual_outputs: pd.Series = self.crossfit.sample_.target
-        assert isinstance(actual_outputs, pd.Series), "sample has one single target"
-
-        return actual_outputs.mean()
+        return self.sample.target.mean()
 
     @staticmethod
     def _expected_pipeline_type() -> Type[RegressorPipelineDF]:
         return RegressorPipelineDF
 
     @staticmethod
-    def _simulate(model: RegressorPipelineDF, x: pd.DataFrame) -> float:
-        return model.predict(X=x).mean(axis=0)
+    def _simulate(model: RegressorPipelineDF, x: pd.DataFrame) -> pd.Series:
+        predictions = model.predict(X=x)
+        assert predictions.ndim == 1, "single-target regressor required"
+        return predictions
 
 
 @inheritdoc(match="[see superclass]")
@@ -617,13 +752,21 @@ class UnivariateUpliftSimulator(_UnivariateRegressionSimulator):
         return 0.0
 
     def simulate_feature(
-        self, feature_name: str, *, partitioner: Partitioner[T_Partition]
+        self,
+        feature_name: str,
+        *,
+        partitioner: Partitioner[T_Partition],
     ) -> UnivariateSimulationResult:
         """[see superclass]"""
         result = super().simulate_feature(
             feature_name=feature_name, partitioner=partitioner
         )
-        result.outputs -= self.expected_output()
+        if self._full_sample:
+            # we only offset the mean values, but not the standard errors of the means
+            # (which are relative values already so don't need to be offset)
+            result.outputs.loc["mean"] -= self.expected_output()
+        else:
+            result.outputs -= self.expected_output()
         return result
 
 
